using System.Collections.Concurrent;
using System.Net;
using System.Text;

namespace Tarpit;

public class ServerUtils
{
    private const string BoilerplateHead = "<!DOCTYPE html>\n\n<html lang=\"en\">\n\n<head>\n\n  <meta charset=\"UTF-8\" />\n\n  <title>Hello, world!</title>\n\n  <meta name=\"viewport\" content=\"width=device-width,initial-scale=1\" />\n\n  <meta name=\"description\" content=\"\" />\n\n  <link rel=\"icon\" href=\"favicon.png\">\n\n</head>";
    private const string BoilerplateBodyStart = "<body>";
    private const string BoilerplateEnd = "</body>\n\n</html>";
    
    public async Task Serve(HttpListenerContext context, ConcurrentQueue<string> tags)
    {
        try
        {
            while (true)
            {
                await ProcessRequestAsync(context, tags);
            }
        }
        catch (HttpListenerException ex)
        {
            Console.WriteLine(ex.Message);
        }
    }

    private async Task ProcessRequestAsync(HttpListenerContext context, ConcurrentQueue<string> tags)
    {
        context.Response.ContentType = "text/html; charset=utf-8";
        Stream outputStream = context.Response.OutputStream;
        
        int i = 0;
        string tag;
        while (true)
        {
            if (tags.IsEmpty)
            {
                continue;
            }
            
            if (i == 0)
            {
                byte[] buffer = Encoding.UTF8.GetBytes(BoilerplateHead);
                await outputStream.WriteAsync(buffer, 0, buffer.Length);
                await outputStream.FlushAsync();
            }
            
            if (i == 1)
            {
                byte[] buffer = Encoding.UTF8.GetBytes(BoilerplateBodyStart);
                await outputStream.WriteAsync(buffer, 0, buffer.Length);
                await outputStream.FlushAsync();
            }
            
            if (tags.TryDequeue(out tag!))
            {
                byte[] buffer = Encoding.UTF8.GetBytes(tag);
                await outputStream.WriteAsync(buffer, 0, buffer.Length);
                await outputStream.FlushAsync();
            }
            
            i++;
            
            Task.Delay(250).Wait();
        }
        
        byte[] temp = Encoding.UTF8.GetBytes(BoilerplateEnd);
        await outputStream.WriteAsync(temp, 0, temp.Length);
        await outputStream.FlushAsync();
        
        outputStream.Close();
    }
}